@@ -1,16 +1,10 @@
 """Store configuration options as a singleton."""
 import os
 import re
-import subprocess
-import sys
 from argparse import Namespace
 from functools import lru_cache
 from typing import Any, Dict, List, Optional, Tuple
 
-from packaging.version import Version
-
-from ansiblelint.constants import ANSIBLE_MISSING_RC
-
 DEFAULT_KINDS = [
     # Do not sort this list, order matters.
     {"jinja2": "**/*.j2"},  # jinja2 templates are not always parsable as something else
@@ -66,11 +60,12 @@ options = Namespace(
     cwd=".",
     display_relative_path=True,
     exclude_paths=[],
+    format="rich",
     lintables=[],
     listrules=False,
     listtags=False,
+    write=False,
     parseable=False,
-    parseable_severity=False,
     quiet=False,
     rulesdirs=[],
     skip_list=[],
@@ -87,7 +82,7 @@ options = Namespace(
     extra_vars=None,
     enable_list=[],
     skip_action_validation=True,
-    rules=dict(),  # Placeholder to set and keep configurations for each rule.
+    rules={},  # Placeholder to set and keep configurations for each rule.
 )
 
 # Used to store detected tag deprecations
@@ -99,9 +94,9 @@ collection_list: List[str] = []
 
 def get_rule_config(rule_id: str) -> Dict[str, Any]:
     """Get configurations for the rule ``rule_id``."""
-    rule_config = options.rules.get(rule_id, dict())
+    rule_config = options.rules.get(rule_id, {})
     if not isinstance(rule_config, dict):
-        raise RuntimeError("Invalid rule config for %s: %s" % (rule_id, rule_config))
+        raise RuntimeError(f"Invalid rule config for {rule_id}: {rule_config}")
     return rule_config
 
 
@@ -112,11 +107,7 @@ def ansible_collections_path() -> str:
     for env_var in ["ANSIBLE_COLLECTIONS_PATHS", "ANSIBLE_COLLECTIONS_PATH"]:
         if env_var in os.environ:
             return env_var
-
-    # https://github.com/ansible/ansible/pull/70007
-    if ansible_version() >= ansible_version("2.10.0.dev0"):
-        return "ANSIBLE_COLLECTIONS_PATH"
-    return "ANSIBLE_COLLECTIONS_PATHS"
+    return "ANSIBLE_COLLECTIONS_PATH"
 
 
 def parse_ansible_version(stdout: str) -> Tuple[str, Optional[str]]:
@@ -131,34 +122,4 @@ def parse_ansible_version(stdout: str) -> Tuple[str, Optional[str]]:
     match = re.search(r"^ansible ([^\s]+)", stdout, re.MULTILINE)
     if match:
         return match.group(1), None
-    return "", "FATAL: Unable parse ansible cli version: %s" % stdout
-
-
-@lru_cache()
-def ansible_version(version: str = "") -> Version:
-    """Return current Version object for Ansible.
-
-    If version is not mentioned, it returns current version as detected.
-    When version argument is mentioned, it return converts the version string
-    to Version object in order to make it usable in comparisons.
-    """
-    if not version:
-        proc = subprocess.run(
-            ["ansible", "--version"],
-            universal_newlines=True,
-            check=False,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-        )
-        if proc.returncode == 0:
-            version, error = parse_ansible_version(proc.stdout)
-            if error is not None:
-                print(error)
-                sys.exit(ANSIBLE_MISSING_RC)
-        else:
-            print(
-                "Unable to find a working copy of ansible executable.",
-                proc,
-            )
-            sys.exit(ANSIBLE_MISSING_RC)
-    return Version(version)
+    return "", f"FATAL: Unable parse ansible cli version: {stdout}"
