@@ -1,11 +1,12 @@
 """Exceptions and error representations."""
 import functools
-from typing import Any, Optional, Union
+from typing import Any, Dict, Optional, Union
 
 from ansiblelint._internal.rules import BaseRule, RuntimeErrorRule
 from ansiblelint.file_utils import Lintable, normpath
 
 
+# pylint: disable=too-many-instance-attributes
 @functools.total_ordering
 class MatchError(ValueError):
     """Rule violation detected during linting.
@@ -46,7 +47,7 @@ class MatchError(ValueError):
 
         self.message = str(message or getattr(rule, "shortdesc", ""))
 
-        # Safety measture to ensure we do not endup with incorrect indexes
+        # Safety measure to ensure we do not end-up with incorrect indexes
         if linenumber == 0:
             raise RuntimeError(
                 "MatchError called incorrectly as line numbers start with 1"
@@ -71,6 +72,11 @@ class MatchError(ValueError):
         # we can still filter by them.
         self.tag = tag
 
+        # optional indicator on how this error was found
+        self.match_type: Optional[str] = None
+        # for task matches, save the normalized task object (useful for transforms)
+        self.task: Optional[Dict[str, Any]] = None
+
     def __repr__(self) -> str:
         """Return a MatchError instance representation."""
         formatstr = "[{0}] ({1}) matched {2}:{3} {4}"
@@ -84,7 +90,7 @@ class MatchError(ValueError):
 
     @property
     def position(self) -> str:
-        """Return error positioniong, with column number if available."""
+        """Return error positioning, with column number if available."""
         if self.column:
             return f"{self.linenumber}:{self.column}"
         return str(self.linenumber)
