@@ -10,7 +10,7 @@ from collections import OrderedDict
 from contextlib import contextmanager
 from pathlib import Path
 from tempfile import NamedTemporaryFile
-from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Set, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Set, Union, cast
 
 # import wcmatch
 import wcmatch.pathlib
@@ -53,10 +53,10 @@ def normpath(path: Union[str, BasePathLike]) -> str:
     """
     # conversion to string in order to allow receiving non string objects
     relpath = os.path.relpath(str(path))
-    abspath = os.path.abspath(str(path))
-    # we avoid returning relative paths that endup at root level
-    if abspath in relpath:
-        return abspath
+    path_absolute = os.path.abspath(str(path))
+    # we avoid returning relative paths that end-up at root level
+    if path_absolute in relpath:
+        return path_absolute
     return relpath
 
 
@@ -122,6 +122,7 @@ def kind_from_path(path: Path, base: bool = False) -> FileType:
     return ""
 
 
+# pylint: disable=too-many-instance-attributes
 class Lintable:
     """Defines a file/folder that can be linted.
 
@@ -147,7 +148,8 @@ class Lintable:
         else:
             self.name = str(name)
             self.path = name
-        self._content = content
+        self._content = self._original_content = content
+        self.updated = False
 
         # if the lintable is part of a role, we save role folder name
         self.role = ""
@@ -197,13 +199,65 @@ class Lintable:
         except NotImplementedError:
             return default
 
+    def _populate_content_cache_from_disk(self) -> None:
+        self._content = self.path.resolve().read_text(encoding="utf-8")
+        if self._original_content is None:
+            self._original_content = self._content
+
     @property
     def content(self) -> str:
-        """Retried file content, from internal cache or disk."""
+        """Retrieve file content, from internal cache or disk."""
         if self._content is None:
-            with open(self.path.resolve(), mode="r", encoding="utf-8") as f:
-                self._content = f.read()
-        return self._content
+            self._populate_content_cache_from_disk()
+        return cast(str, self._content)
+
+    @content.setter
+    def content(self, value: str) -> None:
+        """Update ``content`` and calculate ``updated``.
+
+        To calculate ``updated`` this will read the file from disk if the cache
+        has not already been populated.
+        """
+        if not isinstance(value, str):
+            raise TypeError(f"Expected str but got {type(value)}")
+        if self._original_content is None:
+            if self._content is not None:
+                self._original_content = self._content
+            elif self.path.exists():
+                self._populate_content_cache_from_disk()
+            else:
+                # new file
+                self._original_content = ""
+        self.updated = self._original_content != value
+        self._content = value
+
+    @content.deleter
+    def content(self) -> None:
+        """Reset the internal content cache."""
+        self._content = None
+
+    def write(self, force: bool = False) -> None:
+        """Write the value of ``Lintable.content`` to disk.
+
+        This only writes to disk if the content has been updated (``Lintable.updated``).
+        For example, you can update the content, and then write it to disk like this:
+
+        .. code:: python
+
+            lintable.content = new_content
+            lintable.write()
+
+        Use ``force=True`` when you want to force a content rewrite even if the
+        content has not changed. For example:
+
+        .. code:: python
+
+            lintable.write(force=True)
+        """
+        if not force and not self.updated:
+            # No changes to write.
+            return
+        self.path.resolve().write_text(self._content or "", encoding="utf-8")
 
     def __hash__(self) -> int:
         """Return a hash value of the lintables."""
@@ -220,6 +274,7 @@ class Lintable:
         return f"{self.name} ({self.kind})"
 
 
+# pylint: disable=redefined-outer-name
 def discover_lintables(options: Namespace) -> Dict[str, Any]:
     """Find all files that we know how to lint."""
     # git is preferred as it also considers .gitignore
