@@ -2,6 +2,7 @@
 import copy
 import glob
 import importlib.util
+import inspect
 import logging
 import os
 import re
@@ -20,16 +21,30 @@ from ansiblelint._internal.rules import (
     LoadingFailureRule,
     RuntimeErrorRule,
 )
-from ansiblelint.config import get_rule_config, options
+from ansiblelint.config import get_rule_config
+from ansiblelint.config import options as default_options
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, expand_paths_vars
 
 _logger = logging.getLogger(__name__)
 
+match_types = {
+    "matchlines": "line",
+    "match": "line",  # called by matchlines
+    "matchtasks": "task",
+    "matchtask": "task",  # called by matchtasks
+    "matchyaml": "yaml",
+    "matchplay": "play",  # called by matchyaml
+    "matchdir": "dir",
+}
+
 
 class AnsibleLintRule(BaseRule):
+    """AnsibleLintRule should be used as base for writing new rules."""
+
     @property
     def rule_config(self) -> Dict[str, Any]:
+        """Retrieve rule specific configuration."""
         return get_rule_config(self.id)
 
     @lru_cache()
@@ -43,6 +58,7 @@ class AnsibleLintRule(BaseRule):
 
     @staticmethod
     def unjinja(text: str) -> str:
+        """Remove jinja2 bits from a string."""
         text = re.sub(r"{{.+?}}", "JINJA_EXPRESSION", text)
         text = re.sub(r"{%.+?%}", "JINJA_STATEMENT", text)
         text = re.sub(r"{#.+?#}", "JINJA_COMMENT", text)
@@ -57,6 +73,7 @@ class AnsibleLintRule(BaseRule):
         filename: Optional[Union[str, Lintable]] = None,
         tag: str = "",
     ) -> MatchError:
+        """Instantiate a new MatchError."""
         match = MatchError(
             message=message,
             linenumber=linenumber,
@@ -66,6 +83,17 @@ class AnsibleLintRule(BaseRule):
         )
         if tag:
             match.tag = tag
+        # search through callers to find one of the match* methods
+        frame = inspect.currentframe()
+        match_type: Optional[str] = None
+        while not match_type and frame is not None:
+            func_name = frame.f_code.co_name
+            match_type = match_types.get(func_name, None)
+            if match_type:
+                # add the match_type to the match
+                match.match_type = match_type
+                break
+            frame = frame.f_back  # get the parent frame for the next iteration
         return match
 
     def matchlines(self, file: "Lintable") -> List[MatchError]:
@@ -88,17 +116,15 @@ class AnsibleLintRule(BaseRule):
             message = None
             if isinstance(result, str):
                 message = result
-            m = self.create_matcherror(
+            matcherror = self.create_matcherror(
                 message=message,
                 linenumber=prev_line_no + 1,
                 details=line,
                 filename=file,
             )
-            matches.append(m)
+            matches.append(matcherror)
         return matches
 
-    # TODO(ssbarnea): Reduce mccabe complexity
-    # https://github.com/ansible-community/ansible-lint/issues/744
     def matchtasks(self, file: Lintable) -> List[MatchError]:
         matches: List[MatchError] = []
         if (
@@ -129,13 +155,14 @@ class AnsibleLintRule(BaseRule):
             if isinstance(result, str):
                 message = result
             task_msg = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
-            m = self.create_matcherror(
+            match = self.create_matcherror(
                 message=message,
                 linenumber=task[ansiblelint.utils.LINE_NUMBER_KEY],
                 details=task_msg,
                 filename=file,
             )
-            matches.append(m)
+            match.task = task
+            matches.append(match)
         return matches
 
     def matchyaml(self, file: Lintable) -> List[MatchError]:
@@ -173,9 +200,9 @@ class AnsibleLintRule(BaseRule):
         return matches
 
 
-def is_valid_rule(rule: AnsibleLintRule) -> bool:
+def is_valid_rule(rule: Any) -> bool:
     """Check if given rule is valid or not."""
-    return isinstance(rule, AnsibleLintRule) and bool(rule.id) and bool(rule.shortdesc)
+    return issubclass(rule, AnsibleLintRule) and bool(rule.id) and bool(rule.shortdesc)
 
 
 def load_plugins(directory: str) -> Iterator[AnsibleLintRule]:
@@ -189,17 +216,23 @@ def load_plugins(directory: str) -> Iterator[AnsibleLintRule]:
             module = importlib.util.module_from_spec(spec)
             spec.loader.exec_module(module)
             try:
-                rule = getattr(module, pluginname)()
-                if is_valid_rule(rule):
-                    yield rule
+                for _, obj in inspect.getmembers(module):
+                    if inspect.isclass(obj) and is_valid_rule(obj):
+                        yield obj()
 
-            except (TypeError, ValueError, AttributeError):
-                _logger.warning("Skipped invalid rule from %s", pluginname)
+            except (TypeError, ValueError, AttributeError) as exc:
+                _logger.warning(
+                    "Skipped invalid rule from %s due to %s", pluginname, exc
+                )
 
 
 class RulesCollection:
+    """Container for a collection of rules."""
+
     def __init__(
-        self, rulesdirs: Optional[List[str]] = None, options: Namespace = options
+        self,
+        rulesdirs: Optional[List[str]] = None,
+        options: Namespace = default_options,
     ) -> None:
         """Initialize a RulesCollection instance."""
         self.options = options
@@ -219,6 +252,7 @@ class RulesCollection:
         self.rules = sorted(self.rules)
 
     def register(self, obj: AnsibleLintRule) -> None:
+        """Register a rule."""
         # We skip opt-in rules which were not manually enabled
         if "opt-in" not in obj.tags or obj.id in self.options.enable_list:
             self.rules.append(obj)
@@ -232,24 +266,33 @@ class RulesCollection:
         return len(self.rules)
 
     def extend(self, more: List[AnsibleLintRule]) -> None:
+        """Combine rules."""
         self.rules.extend(more)
 
     def run(
-        self, file: Lintable, tags: Set[str] = set(), skip_list: List[str] = []
+        self,
+        file: Lintable,
+        tags: Optional[Set[str]] = None,
+        skip_list: Optional[List[str]] = None,
     ) -> List[MatchError]:
-        matches: List[MatchError] = list()
+        """Run all the rules against the given lintable."""
+        matches: List[MatchError] = []
+        if tags is None:
+            tags = set()
+        if skip_list is None:
+            skip_list = []
 
         if not file.path.is_dir():
             try:
                 if file.content is not None:  # loads the file content
                     pass
-            except IOError as e:
+            except IOError as exc:
                 return [
                     MatchError(
-                        message=str(e),
+                        message=str(exc),
                         filename=file,
                         rule=LoadingFailureRule(),
-                        tag=e.__class__.__name__.lower(),
+                        tag=exc.__class__.__name__.lower(),
                     )
                 ]
 
@@ -277,8 +320,8 @@ class RulesCollection:
         )
 
     def listtags(self) -> str:
+        """Return a string with all the tags in the RulesCollection."""
         tag_desc = {
-            "behaviour": "Indicates a bad practice or behavior",
             "command-shell": "Specific to use of command and shell modules",
             "core": "Related to internal implementation of the linter",
             "deprecations": "Indicate use of features that are removed from Ansible",
@@ -287,7 +330,11 @@ class RulesCollection:
             "idempotency": "Possible indication that consequent runs would produce different results",
             "idiom": "Anti-pattern detected, likely to cause undesired behavior",
             "metadata": "Invalid metadata, likely related to galaxy, collections or roles",
-            "yaml": "External linter which will also produce its own rule codes.",
+            "opt-in": "Rules that are not used unless manually added to `enable_list`",
+            "security": "Rules related o potentially security issues, like exposing credentials",
+            "unpredictability": "Warn about code that might not work in a predictable way",
+            "unskippable": "Indicate a fatal error that cannot be ignored or disabled",
+            "yaml": "External linter which will also produce its own rule codes",
         }
 
         tags = defaultdict(list)
